// Copyright 2022 Trillian Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package crdb

import (
	"fmt"
	"strings"
)

func fromMySQLToPGPreparedStatement(sql string) string {
	counter := 1
	for strings.Contains(sql, "?") {
		pgmarker := fmt.Sprintf("$%d", counter)
		sql = strings.Replace(sql, "?", pgmarker, 1)
		counter++
	}
	return sql
}
