// Copyright 2023 Google LLC. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";
option go_package = "github.com/google/trillian/storage/mysql/mysqlpb";

package mysqlpb;

// StorageOptions contains configuration parameters for MySQL implementation
// of the storage backend. This is envisioned only to be used for changes that
// would be breaking, but need to support old behaviour for backwards compatibility.
message StorageOptions {
    // subtreeRevisions being explicitly set to false will skip writing subtree revisions.
    // https://github.com/google/trillian/pull/3201
    bool subtreeRevisions = 1;
}
