// Copyright 2017 Google LLC. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package prometheus

import (
	"testing"

	"github.com/google/trillian/monitoring/testonly"
)

func TestCounter(t *testing.T) {
	testonly.TestCounter(t, MetricFactory{Prefix: "TestCounter"})
}

func TestGauge(t *testing.T) {
	testonly.TestGauge(t, MetricFactory{Prefix: "TestGauge"})
}

func TestHistogram(t *testing.T) {
	testonly.TestHistogram(t, MetricFactory{Prefix: "TestHistogram"})
}
