// Code generated by "stringer -type=OpType types.go"; DO NOT EDIT.

package trees

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[Unknown-0]
	_ = x[Admin-1]
	_ = x[Query-2]
	_ = x[QueueLog-3]
	_ = x[SequenceLog-4]
	_ = x[UpdateMap-5]
}

const _OpType_name = "UnknownAdminQueryQueueLogSequenceLogUpdateMap"

var _OpType_index = [...]uint8{0, 7, 12, 17, 25, 36, 45}

func (i OpType) String() string {
	if i < 0 || i >= OpType(len(_OpType_index)-1) {
		return "OpType(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _OpType_name[_OpType_index[i]:_OpType_index[i+1]]
}
